@@ -13,7 +13,6 @@ import subprocess
 import sys
 import threading
 import errno
-import unittest
 import weakref
 
 try:
@@ -34,8 +33,10 @@ from trollius import compat
 from trollius import events
 from trollius import proactor_events
 from trollius import selector_events
+from trollius import sslproto
 from trollius import test_support as support
 from trollius import test_utils
+from trollius.test_utils import unittest
 from trollius.py33_exceptions import (wrap_error,
     BlockingIOError, ConnectionRefusedError,
     FileNotFoundError)
@@ -63,6 +64,11 @@ def osx_tiger():
     return version < (10, 5)
 
 
+def skip_if_backported_sslcontext():
+    backported = getattr(asyncio, 'BACKPORT_SSL_CONTEXT', False)
+    return unittest.skipIf(backported, 'need ssl.SSLContext')
+
+
 ONLYCERT = data_file('ssl_cert.pem')
 ONLYKEY = data_file('ssl_key.pem')
 SIGNED_CERTFILE = data_file('keycert3.pem')
@@ -342,7 +348,7 @@ class EventLoopTestsMixin(object):
         self.loop.run_forever()
         self.assertEqual(results, ['hello', 'world'])
 
-    @test_utils.skipIf(concurrent is None, 'need concurrent.futures')
+    @unittest.skipIf(concurrent is None, 'need concurrent.futures')
     def test_run_in_executor(self):
         def run(arg):
             return (arg, threading.current_thread().ident)
@@ -437,7 +443,7 @@ class EventLoopTestsMixin(object):
             sock = socket.socket()
             self._basetest_sock_client_ops(httpd, sock)
 
-    @test_utils.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
+    @unittest.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
     def test_unix_sock_client_ops(self):
         with test_utils.run_test_unix_server() as httpd:
             sock = socket.socket(socket.AF_UNIX)
@@ -477,7 +483,7 @@ class EventLoopTestsMixin(object):
         conn.close()
         listener.close()
 
-    @test_utils.skipUnless(hasattr(signal, 'SIGKILL'), 'No SIGKILL')
+    @unittest.skipUnless(hasattr(signal, 'SIGKILL'), 'No SIGKILL')
     def test_add_signal_handler(self):
         non_local = {'caught': 0}
 
@@ -520,7 +526,7 @@ class EventLoopTestsMixin(object):
         # Removing again returns False.
         self.assertFalse(self.loop.remove_signal_handler(signal.SIGINT))
 
-    @test_utils.skipUnless(hasattr(signal, 'SIGALRM'), 'No SIGALRM')
+    @unittest.skipUnless(hasattr(signal, 'SIGALRM'), 'No SIGALRM')
     def test_signal_handling_while_selecting(self):
         # Test with a signal actually arriving during a select() call.
         non_local = {'caught': 0}
@@ -535,7 +541,7 @@ class EventLoopTestsMixin(object):
         self.loop.run_forever()
         self.assertEqual(non_local['caught'], 1)
 
-    @test_utils.skipUnless(hasattr(signal, 'SIGALRM'), 'No SIGALRM')
+    @unittest.skipUnless(hasattr(signal, 'SIGALRM'), 'No SIGALRM')
     def test_signal_handling_args(self):
         some_args = (42,)
         non_local = {'caught': 0}
@@ -568,7 +574,7 @@ class EventLoopTestsMixin(object):
                 lambda: MyProto(loop=self.loop), *httpd.address)
             self._basetest_create_connection(conn_fut)
 
-    @test_utils.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
+    @unittest.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
     def test_create_unix_connection(self):
         # Issue #20682: On Mac OS X Tiger, getsockname() returns a
         # zero-length address for UNIX socket.
@@ -625,7 +631,6 @@ class EventLoopTestsMixin(object):
         self._basetest_create_ssl_connection(conn_fut, check_sockname)
 
         # ssl.Purpose was introduced in Python 3.4
-        #if not asyncio.BACKPORT_SSL_CONTEXT:
         if hasattr(ssl, 'Purpose'):
             def _dummy_ssl_create_context(purpose=ssl.Purpose.SERVER_AUTH,
                                           cafile=None, capath=None,
@@ -653,7 +658,11 @@ class EventLoopTestsMixin(object):
                 with test_utils.disable_logger():
                     self._basetest_create_ssl_connection(conn_fut, check_sockname)
 
-    @test_utils.skipIf(ssl is None, 'No ssl module')
+            # Test for Python 3.2
+            if hasattr(ssl.SSLError, 'reason'):
+                self.assertEqual(cm.exception.reason, 'CERTIFICATE_VERIFY_FAILED')
+
+    @unittest.skipIf(ssl is None, 'No ssl module')
     def test_create_ssl_connection(self):
         with test_utils.run_test_server(use_ssl=True) as httpd:
             create_connection = functools.partial(
@@ -662,8 +671,12 @@ class EventLoopTestsMixin(object):
                 *httpd.address)
             self._test_create_ssl_connection(httpd, create_connection)
 
-    @test_utils.skipIf(ssl is None, 'No ssl module')
-    @test_utils.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
+    def test_legacy_create_ssl_connection(self):
+        with test_utils.force_legacy_ssl_support():
+            self.test_create_ssl_connection()
+
+    @unittest.skipIf(ssl is None, 'No ssl module')
+    @unittest.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
     def test_create_ssl_unix_connection(self):
         # Issue #20682: On Mac OS X Tiger, getsockname() returns a
         # zero-length address for UNIX socket.
@@ -678,6 +691,10 @@ class EventLoopTestsMixin(object):
             self._test_create_ssl_connection(httpd, create_connection,
                                              check_sockname)
 
+    def test_legacy_create_ssl_unix_connection(self):
+        with test_utils.force_legacy_ssl_support():
+            self.test_create_ssl_unix_connection()
+
     def test_create_connection_local_addr(self):
         with test_utils.run_test_server() as httpd:
             port = support.find_unused_port()
@@ -745,7 +762,7 @@ class EventLoopTestsMixin(object):
 
         return server, path
 
-    @test_utils.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
+    @unittest.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
     def test_create_unix_server(self):
         proto = MyProto(loop=self.loop)
         server, path = self._make_unix_server(lambda: proto)
@@ -773,7 +790,7 @@ class EventLoopTestsMixin(object):
         # close server
         server.close()
 
-    @test_utils.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
+    @unittest.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
     def test_create_unix_server_path_socket_error(self):
         proto = MyProto(loop=self.loop)
         sock = socket.socket()
@@ -808,7 +825,7 @@ class EventLoopTestsMixin(object):
         sslcontext = self._create_ssl_context(certfile, keyfile)
         return self._make_unix_server(factory, ssl=sslcontext)
 
-    @test_utils.skipIf(ssl is None, 'No ssl module')
+    @unittest.skipIf(ssl is None, 'No ssl module')
     def test_create_server_ssl(self):
         proto = MyProto(loop=self.loop)
         server, host, port = self._make_ssl_server(
@@ -842,8 +859,12 @@ class EventLoopTestsMixin(object):
         # stop serving
         server.close()
 
-    @test_utils.skipIf(ssl is None, 'No ssl module')
-    @test_utils.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
+    def test_legacy_create_server_ssl(self):
+        with test_utils.force_legacy_ssl_support():
+            self.test_create_server_ssl()
+
+    @unittest.skipIf(ssl is None, 'No ssl module')
+    @unittest.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
     def test_create_unix_server_ssl(self):
         proto = MyProto(loop=self.loop)
         server, path = self._make_ssl_unix_server(
@@ -873,8 +894,12 @@ class EventLoopTestsMixin(object):
         # stop serving
         server.close()
 
-    @test_utils.skipIf(ssl is None, 'No ssl module')
-    @test_utils.skipIf(asyncio.BACKPORT_SSL_CONTEXT, 'need ssl.SSLContext')
+    def test_legacy_create_unix_server_ssl(self):
+        with test_utils.force_legacy_ssl_support():
+            self.test_create_unix_server_ssl()
+
+    @unittest.skipIf(ssl is None, 'No ssl module')
+    @skip_if_backported_sslcontext()
     def test_create_server_ssl_verify_failed(self):
         proto = MyProto(loop=self.loop)
         server, host, port = self._make_ssl_server(
@@ -889,18 +914,26 @@ class EventLoopTestsMixin(object):
         # no CA loaded
         f_c = self.loop.create_connection(MyProto, host, port,
                                           ssl=sslcontext_client)
-        with test_utils.disable_logger():
-            with self.assertRaisesRegex(ssl.SSLError,
-                                        'certificate verify failed'):
-                self.loop.run_until_complete(f_c)
+        with mock.patch.object(self.loop, 'call_exception_handler'):
+            with test_utils.disable_logger():
+                with self.assertRaisesRegex(ssl.SSLError,
+                                            'certificate verify failed'):
+                    self.loop.run_until_complete(f_c)
+
+            # execute the loop to log the connection error
+            test_utils.run_briefly(self.loop)
 
         # close connection
         self.assertIsNone(proto.transport)
         server.close()
 
-    @test_utils.skipIf(ssl is None, 'No ssl module')
-    @test_utils.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
-    @test_utils.skipIf(asyncio.BACKPORT_SSL_CONTEXT, 'need ssl.SSLContext')
+    def test_legacy_create_server_ssl_verify_failed(self):
+        with test_utils.force_legacy_ssl_support():
+            self.test_create_server_ssl_verify_failed()
+
+    @unittest.skipIf(ssl is None, 'No ssl module')
+    @unittest.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
+    @skip_if_backported_sslcontext()
     def test_create_unix_server_ssl_verify_failed(self):
         proto = MyProto(loop=self.loop)
         server, path = self._make_ssl_unix_server(
@@ -916,16 +949,24 @@ class EventLoopTestsMixin(object):
         f_c = self.loop.create_unix_connection(MyProto, path,
                                                ssl=sslcontext_client,
                                                server_hostname='invalid')
-        with test_utils.disable_logger():
-            with self.assertRaisesRegex(ssl.SSLError,
-                                        'certificate verify failed'):
-                self.loop.run_until_complete(f_c)
+        with mock.patch.object(self.loop, 'call_exception_handler'):
+            with test_utils.disable_logger():
+                with self.assertRaisesRegex(ssl.SSLError,
+                                            'certificate verify failed'):
+                    self.loop.run_until_complete(f_c)
+
+            # execute the loop to log the connection error
+            test_utils.run_briefly(self.loop)
 
         # close connection
         self.assertIsNone(proto.transport)
         server.close()
 
-    @test_utils.skipIf(ssl is None, 'No ssl module')
+    def test_legacy_create_unix_server_ssl_verify_failed(self):
+        with test_utils.force_legacy_ssl_support():
+            self.test_create_unix_server_ssl_verify_failed()
+
+    @unittest.skipIf(ssl is None, 'No ssl module')
     def test_create_server_ssl_match_failed(self):
         proto = MyProto(loop=self.loop)
         server, host, port = self._make_ssl_server(
@@ -950,19 +991,24 @@ class EventLoopTestsMixin(object):
         if not asyncio.BACKPORT_SSL_CONTEXT:
             f_c = self.loop.create_connection(MyProto, host, port,
                                               ssl=sslcontext_client)
-            with test_utils.disable_logger():
-                with self.assertRaisesRegex(
-                        ssl.CertificateError,
-                        err_msg):
-                    self.loop.run_until_complete(f_c)
+            with mock.patch.object(self.loop, 'call_exception_handler'):
+                with test_utils.disable_logger():
+                    with self.assertRaisesRegex(
+                            ssl.CertificateError,
+                            err_msg):
+                        self.loop.run_until_complete(f_c)
 
             # close connection
             proto.transport.close()
 
         server.close()
 
-    @test_utils.skipIf(ssl is None, 'No ssl module')
-    @test_utils.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
+    def test_legacy_create_server_ssl_match_failed(self):
+        with test_utils.force_legacy_ssl_support():
+            self.test_create_server_ssl_match_failed()
+
+    @unittest.skipIf(ssl is None, 'No ssl module')
+    @unittest.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
     def test_create_unix_server_ssl_verified(self):
         proto = MyProto(loop=self.loop)
         server, path = self._make_ssl_unix_server(
@@ -986,8 +1032,13 @@ class EventLoopTestsMixin(object):
         proto.transport.close()
         client.close()
         server.close()
+        self.loop.run_until_complete(proto.done)
 
-    @test_utils.skipIf(ssl is None, 'No ssl module')
+    def test_legacy_create_unix_server_ssl_verified(self):
+        with test_utils.force_legacy_ssl_support():
+            self.test_create_unix_server_ssl_verified()
+
+    @unittest.skipIf(ssl is None, 'No ssl module')
     def test_create_server_ssl_verified(self):
         proto = MyProto(loop=self.loop)
         server, host, port = self._make_ssl_server(
@@ -1001,18 +1052,22 @@ class EventLoopTestsMixin(object):
         if hasattr(sslcontext_client, 'check_hostname'):
             sslcontext_client.check_hostname = True
 
-        if not asyncio.BACKPORT_SSL_CONTEXT:
-            # Connection succeeds with correct CA and server hostname.
-            f_c = self.loop.create_connection(MyProto, host, port,
-                                              ssl=sslcontext_client,
-                                              server_hostname='localhost')
-            client, pr = self.loop.run_until_complete(f_c)
+        # Connection succeeds with correct CA and server hostname.
+        f_c = self.loop.create_connection(MyProto, host, port,
+                                          ssl=sslcontext_client,
+                                          server_hostname='localhost')
+        client, pr = self.loop.run_until_complete(f_c)
 
-            # close connection
-            proto.transport.close()
-            client.close()
+        # close connection
+        proto.transport.close()
+        client.close()
 
         server.close()
+        self.loop.run_until_complete(proto.done)
+
+    def test_legacy_create_server_ssl_verified(self):
+        with test_utils.force_legacy_ssl_support():
+            self.test_create_server_ssl_verified()
 
     def test_create_server_sock(self):
         non_local = {'proto': asyncio.Future(loop=self.loop)}
@@ -1056,7 +1111,7 @@ class EventLoopTestsMixin(object):
 
         server.close()
 
-    @test_utils.skipUnless(support.IPV6_ENABLED, 'IPv6 not supported or enabled')
+    @unittest.skipUnless(support.IPV6_ENABLED, 'IPv6 not supported or enabled')
     def test_create_server_dual_stack(self):
         f_proto = asyncio.Future(loop=self.loop)
 
@@ -1107,6 +1162,7 @@ class EventLoopTestsMixin(object):
         client.connect(('127.0.0.1', port))
         client.send(b'xxx')
         client.close()
+
         server.close()
 
         client = socket.socket()
@@ -1173,7 +1229,7 @@ class EventLoopTestsMixin(object):
         self.assertIsNone(loop._csock)
         self.assertIsNone(loop._ssock)
 
-    @test_utils.skipUnless(sys.platform != 'win32',
+    @unittest.skipUnless(sys.platform != 'win32',
                          "Don't support pipes for Windows")
     def test_read_pipe(self):
         proto = MyReadPipeProto(loop=self.loop)
@@ -1208,7 +1264,7 @@ class EventLoopTestsMixin(object):
         # extra info is available
         self.assertIsNotNone(proto.transport.get_extra_info('pipe'))
 
-    @test_utils.skipUnless(sys.platform != 'win32',
+    @unittest.skipUnless(sys.platform != 'win32',
                          "Don't support pipes for Windows")
     # select, poll and kqueue don't support character devices (PTY) on Mac OS X
     # older than 10.6 (Snow Leopard)
@@ -1248,8 +1304,8 @@ class EventLoopTestsMixin(object):
         # extra info is available
         self.assertIsNotNone(proto.transport.get_extra_info('pipe'))
 
-    @test_utils.skipUnless(sys.platform != 'win32',
-                           "Don't support pipes for Windows")
+    @unittest.skipUnless(sys.platform != 'win32',
+                         "Don't support pipes for Windows")
     def test_write_pipe(self):
         rpipe, wpipe = os.pipe()
         pipeobj = io.open(wpipe, 'wb', 1024)
@@ -1287,7 +1343,7 @@ class EventLoopTestsMixin(object):
         self.loop.run_until_complete(proto.done)
         self.assertEqual('CLOSED', proto.state)
 
-    @test_utils.skipUnless(sys.platform != 'win32',
+    @unittest.skipUnless(sys.platform != 'win32',
                          "Don't support pipes for Windows")
     def test_write_pipe_disconnect_on_close(self):
         rsock, wsock = test_utils.socketpair()
@@ -1315,8 +1371,8 @@ class EventLoopTestsMixin(object):
         self.loop.run_until_complete(proto.done)
         self.assertEqual('CLOSED', proto.state)
 
-    @test_utils.skipUnless(sys.platform != 'win32',
-                           "Don't support pipes for Windows")
+    @unittest.skipUnless(sys.platform != 'win32',
+                         "Don't support pipes for Windows")
     # select, poll and kqueue don't support character devices (PTY) on Mac OS X
     # older than 10.6 (Snow Leopard)
     @support.requires_mac_ver(10, 6)
@@ -1424,6 +1480,10 @@ class EventLoopTestsMixin(object):
              'selector': self.loop._selector.__class__.__name__})
 
     def test_sock_connect_address(self):
+        # In debug mode, sock_connect() must ensure that the address is already
+        # resolved (call _check_resolved_address())
+        self.loop.set_debug(True)
+
         addresses = [(socket.AF_INET, ('www.python.org', 80))]
         if support.IPV6_ENABLED:
             addresses.extend((
@@ -1538,9 +1598,10 @@ class SubprocessTestsMixin(object):
         stdin = transp.get_pipe_transport(0)
         stdin.write(b'Python The Winner')
         self.loop.run_until_complete(proto.got_data[1].wait())
-        transp.close()
+        with test_utils.disable_logger():
+            transp.close()
         self.loop.run_until_complete(proto.completed)
-        self.check_terminated(proto.returncode)
+        self.check_killed(proto.returncode)
         self.assertEqual(b'Python The Winner', proto.data[1])
 
     def test_subprocess_interactive(self):
@@ -1554,21 +1615,20 @@ class SubprocessTestsMixin(object):
         self.loop.run_until_complete(proto.connected)
         self.assertEqual('CONNECTED', proto.state)
 
-        try:
-            stdin = transp.get_pipe_transport(0)
-            stdin.write(b'Python ')
-            self.loop.run_until_complete(proto.got_data[1].wait())
-            proto.got_data[1].clear()
-            self.assertEqual(b'Python ', proto.data[1])
+        stdin = transp.get_pipe_transport(0)
+        stdin.write(b'Python ')
+        self.loop.run_until_complete(proto.got_data[1].wait())
+        proto.got_data[1].clear()
+        self.assertEqual(b'Python ', proto.data[1])
 
-            stdin.write(b'The Winner')
-            self.loop.run_until_complete(proto.got_data[1].wait())
-            self.assertEqual(b'Python The Winner', proto.data[1])
-        finally:
+        stdin.write(b'The Winner')
+        self.loop.run_until_complete(proto.got_data[1].wait())
+        self.assertEqual(b'Python The Winner', proto.data[1])
+
+        with test_utils.disable_logger():
             transp.close()
-
         self.loop.run_until_complete(proto.completed)
-        self.check_terminated(proto.returncode)
+        self.check_killed(proto.returncode)
 
     def test_subprocess_shell(self):
         connect = self.loop.subprocess_shell(
@@ -1584,6 +1644,7 @@ class SubprocessTestsMixin(object):
         self.assertTrue(all(f.done() for f in proto.disconnects.values()))
         self.assertEqual(proto.data[1].rstrip(b'\r\n'), b'Python')
         self.assertEqual(proto.data[2], b'')
+        transp.close()
 
     def test_subprocess_exitcode(self):
         connect = self.loop.subprocess_shell(
@@ -1593,6 +1654,7 @@ class SubprocessTestsMixin(object):
         self.assertIsInstance(proto, MySubprocessProtocol)
         self.loop.run_until_complete(proto.completed)
         self.assertEqual(7, proto.returncode)
+        transp.close()
 
     def test_subprocess_close_after_finish(self):
         connect = self.loop.subprocess_shell(
@@ -1620,6 +1682,7 @@ class SubprocessTestsMixin(object):
         transp.kill()
         self.loop.run_until_complete(proto.completed)
         self.check_killed(proto.returncode)
+        transp.close()
 
     def test_subprocess_terminate(self):
         prog = os.path.join(os.path.dirname(__file__), 'echo.py')
@@ -1634,8 +1697,9 @@ class SubprocessTestsMixin(object):
         transp.terminate()
         self.loop.run_until_complete(proto.completed)
         self.check_terminated(proto.returncode)
+        transp.close()
 
-    @test_utils.skipIf(sys.platform == 'win32', "Don't have SIGHUP")
+    @unittest.skipIf(sys.platform == 'win32', "Don't have SIGHUP")
     def test_subprocess_send_signal(self):
         prog = os.path.join(os.path.dirname(__file__), 'echo.py')
 
@@ -1649,6 +1713,7 @@ class SubprocessTestsMixin(object):
         transp.send_signal(signal.SIGHUP)
         self.loop.run_until_complete(proto.completed)
         self.assertEqual(-signal.SIGHUP, proto.returncode)
+        transp.close()
 
     def test_subprocess_stderr(self):
         prog = os.path.join(os.path.dirname(__file__), 'echo2.py')
@@ -1721,11 +1786,12 @@ class SubprocessTestsMixin(object):
             # GetLastError()==ERROR_INVALID_NAME on Windows!?!  (Using
             # WriteFile() we get ERROR_BROKEN_PIPE as expected.)
             self.assertEqual(b'ERR:OSError', proto.data[2])
-        transp.close()
+        with test_utils.disable_logger():
+            transp.close()
         self.loop.run_until_complete(proto.completed)
-        self.check_terminated(proto.returncode)
+        self.check_killed(proto.returncode)
 
-    @test_utils.skipUnless(hasattr(os, 'setsid'), "need os.setsid()")
+    @unittest.skipUnless(hasattr(os, 'setsid'), "need os.setsid()")
     def test_subprocess_wait_no_same_group(self):
         # start the new process in a new session
         connect = self.loop.subprocess_shell(
@@ -1784,20 +1850,36 @@ if sys.platform == 'win32':
         def create_event_loop(self):
             return asyncio.ProactorEventLoop()
 
-        def test_create_ssl_connection(self):
-            raise unittest.SkipTest("IocpEventLoop incompatible with SSL")
+        if not sslproto._is_sslproto_available():
+            def test_create_ssl_connection(self):
+                raise unittest.SkipTest("need python 3.5 (ssl.MemoryBIO)")
 
-        def test_create_server_ssl(self):
-            raise unittest.SkipTest("IocpEventLoop incompatible with SSL")
+            def test_create_server_ssl(self):
+                raise unittest.SkipTest("need python 3.5 (ssl.MemoryBIO)")
 
-        def test_create_server_ssl_verify_failed(self):
-            raise unittest.SkipTest("IocpEventLoop incompatible with SSL")
+            def test_create_server_ssl_verify_failed(self):
+                raise unittest.SkipTest("need python 3.5 (ssl.MemoryBIO)")
 
-        def test_create_server_ssl_match_failed(self):
-            raise unittest.SkipTest("IocpEventLoop incompatible with SSL")
+            def test_create_server_ssl_match_failed(self):
+                raise unittest.SkipTest("need python 3.5 (ssl.MemoryBIO)")
 
-        def test_create_server_ssl_verified(self):
-            raise unittest.SkipTest("IocpEventLoop incompatible with SSL")
+            def test_create_server_ssl_verified(self):
+                raise unittest.SkipTest("need python 3.5 (ssl.MemoryBIO)")
+
+        def test_legacy_create_ssl_connection(self):
+            raise unittest.SkipTest("IocpEventLoop incompatible with legacy SSL")
+
+        def test_legacy_create_server_ssl(self):
+            raise unittest.SkipTest("IocpEventLoop incompatible with legacy SSL")
+
+        def test_legacy_create_server_ssl_verify_failed(self):
+            raise unittest.SkipTest("IocpEventLoop incompatible with legacy SSL")
+
+        def test_legacy_create_server_ssl_match_failed(self):
+            raise unittest.SkipTest("IocpEventLoop incompatible with legacy SSL")
+
+        def test_legacy_create_server_ssl_verified(self):
+            raise unittest.SkipTest("IocpEventLoop incompatible with legacy SSL")
 
         def test_reader_callback(self):
             raise unittest.SkipTest("IocpEventLoop does not have add_reader()")
@@ -1845,8 +1927,8 @@ else:
             @support.requires_mac_ver(10, 9)
             # Issue #20667: KqueueEventLoopTests.test_read_pty_output()
             # hangs on OpenBSD 5.5
-            @test_utils.skipIf(sys.platform.startswith('openbsd'),
-                              'test hangs on OpenBSD')
+            @unittest.skipIf(sys.platform.startswith('openbsd'),
+                             'test hangs on OpenBSD')
             def test_read_pty_output(self):
                 super(KqueueEventLoopTests, self).test_read_pty_output()
 
@@ -2006,21 +2088,30 @@ class HandleTests(test_utils.TestCase):
         loop.set_debug(True)
         self.set_event_loop(loop)
 
+        def check_source_traceback(h):
+            lineno = sys._getframe(1).f_lineno - 1
+            self.assertIsInstance(h._source_traceback, list)
+            filename = sys._getframe().f_code.co_filename
+            self.assertEqual(h._source_traceback[-1][:3],
+                             (filename,
+                              lineno,
+                              'test_handle_source_traceback'))
+
         # call_soon
         h = loop.call_soon(noop)
-        self.check_soure_traceback(h._source_traceback, -1)
+        check_source_traceback(h)
 
         # call_soon_threadsafe
         h = loop.call_soon_threadsafe(noop)
-        self.check_soure_traceback(h._source_traceback, -1)
+        check_source_traceback(h)
 
         # call_later
         h = loop.call_later(0, noop)
-        self.check_soure_traceback(h._source_traceback, -1)
+        check_source_traceback(h)
 
         # call_at
         h = loop.call_later(0, noop)
-        self.check_soure_traceback(h._source_traceback, -1)
+        check_source_traceback(h)
 
 
 class TimerTests(test_utils.TestCase):
