@@ -4,13 +4,14 @@ from __future__ import print_function
 __all__ = ['Task',
            'FIRST_COMPLETED', 'FIRST_EXCEPTION', 'ALL_COMPLETED',
            'wait', 'wait_for', 'as_completed', 'sleep', 'async',
-           'gather', 'shield', 'Return', 'From',
+           'gather', 'shield', 'ensure_future',
            ]
 
 import functools
 import linecache
 import sys
 import traceback
+import warnings
 try:
     from weakref import WeakSet
 except ImportError:
@@ -26,8 +27,6 @@ from .locks import Lock, Condition, Semaphore, _ContextManager
 from .coroutines import coroutine, From, Return
 
 
-_PY34 = (sys.version_info >= (3, 4))
-
 
 @coroutine
 def _lock_coroutine(lock):
@@ -81,11 +80,11 @@ class Task(futures.Future):
         return set(t for t in cls._all_tasks if t._loop is loop)
 
     def __init__(self, coro, loop=None):
-        assert coroutines.iscoroutine(coro), repr(coro)  # Not a coroutine function!
+        assert coroutines.iscoroutine(coro), repr(coro)
         super(Task, self).__init__(loop=loop)
         if self._source_traceback:
             del self._source_traceback[-1]
-        self._coro = iter(coro)  # Use the iterator just in case.
+        self._coro = coro
         self._fut_waiter = None
         self._must_cancel = False
         self._loop.call_soon(self._step)
@@ -94,7 +93,7 @@ class Task(futures.Future):
     # On Python 3.3 or older, objects with a destructor that are part of a
     # reference cycle are never destroyed. That's not the case any more on
     # Python 3.4 thanks to the PEP 442.
-    if _PY34:
+    if compat.PY34:
         def __del__(self):
             if self._state == futures._PENDING and self._log_destroy_pending:
                 context = {
@@ -249,10 +248,8 @@ class Task(futures.Future):
         try:
             if exc is not None:
                 result = coro.throw(exc)
-            elif value is not None:
-                result = coro.send(value)
             else:
-                result = next(coro)
+                result = coro.send(value)
         except StopIteration as exc:
             if compat.PY33:
                 # asyncio Task object? get the result of the coroutine
@@ -344,7 +341,7 @@ class Task(futures.Future):
 
 # wait() and as_completed() similar to those in PEP 3148.
 
-# Export symbols in trollius.tasks for compatibility with Tulip
+# Export symbols in trollius.tasks for compatibility with asyncio
 FIRST_COMPLETED = executor.FIRST_COMPLETED
 FIRST_EXCEPTION = executor.FIRST_EXCEPTION
 ALL_COMPLETED = executor.ALL_COMPLETED
@@ -362,7 +359,7 @@ def wait(fs, loop=None, timeout=None, return_when=ALL_COMPLETED):
 
     Usage:
 
-        done, pending = yield From(trollius.wait(fs))
+        done, pending = yield From(asyncio.wait(fs))
 
     Note: This does not raise TimeoutError! Futures that aren't done
     when the timeout occurs are returned in the second set.
@@ -377,7 +374,7 @@ def wait(fs, loop=None, timeout=None, return_when=ALL_COMPLETED):
     if loop is None:
         loop = events.get_event_loop()
 
-    fs = set(async(f, loop=loop) for f in set(fs))
+    fs = set(ensure_future(f, loop=loop) for f in set(fs))
 
     result = yield From(_wait(fs, timeout, return_when, loop))
     raise Return(result)
@@ -398,27 +395,32 @@ def wait_for(fut, timeout, loop=None):
     it cancels the task and raises TimeoutError.  To avoid the task
     cancellation, wrap it in shield().
 
-    Usage:
-
-        result = yield From(trollius.wait_for(fut, 10.0))
+    If the wait is cancelled, the task is also cancelled.
 
+    This function is a coroutine.
     """
     if loop is None:
         loop = events.get_event_loop()
 
     if timeout is None:
-        raise Return((yield From(fut)))
+        result = yield From(fut)
+        raise Return(result)
 
     waiter = futures.Future(loop=loop)
     timeout_handle = loop.call_later(timeout, _release_waiter, waiter)
     cb = functools.partial(_release_waiter, waiter)
 
-    fut = async(fut, loop=loop)
+    fut = ensure_future(fut, loop=loop)
     fut.add_done_callback(cb)
 
     try:
         # wait until the future completes or the timeout
-        yield From(waiter)
+        try:
+            yield From(waiter)
+        except futures.CancelledError:
+            fut.remove_done_callback(cb)
+            fut.cancel()
+            raise
 
         if fut.done():
             raise Return(fut.result())
@@ -495,7 +497,7 @@ def as_completed(fs, loop=None, timeout=None):
     if isinstance(fs, futures._FUTURE_CLASSES) or coroutines.iscoroutine(fs):
         raise TypeError("expect a list of futures, not %s" % type(fs).__name__)
     loop = loop if loop is not None else events.get_event_loop()
-    todo = set(async(f, loop=loop) for f in set(fs))
+    todo = set(ensure_future(f, loop=loop) for f in set(fs))
     from .queues import Queue  # Import here to avoid circular import problem.
     done = Queue(loop=loop)
     timeout_handle = None
@@ -546,6 +548,20 @@ def sleep(delay, result=None, loop=None):
 def async(coro_or_future, loop=None):
     """Wrap a coroutine in a future.
 
+    If the argument is a Future, it is returned directly.
+
+    This function is deprecated in 3.5. Use asyncio.ensure_future() instead.
+    """
+
+    warnings.warn("asyncio.async() function is deprecated, use ensure_future()",
+                  DeprecationWarning)
+
+    return ensure_future(coro_or_future, loop=loop)
+
+
+def ensure_future(coro_or_future, loop=None):
+    """Wrap a coroutine in a future.
+
     If the argument is a Future, it is returned directly.
     """
     # FIXME: only check if coroutines._DEBUG is True?
@@ -619,7 +635,7 @@ def gather(*coros_or_futures, **kw):
     arg_to_fut = {}
     for arg in set(coros_or_futures):
         if not isinstance(arg, futures._FUTURE_CLASSES):
-            fut = async(arg, loop=loop)
+            fut = ensure_future(arg, loop=loop)
             if loop is None:
                 loop = fut._loop
             # The caller cannot control this future, the "destroy pending task"
@@ -640,12 +656,13 @@ def gather(*coros_or_futures, **kw):
     results = [None] * nchildren
 
     def _done_callback(i, fut):
-        if outer._state != futures._PENDING:
-            if fut._exception is not None:
+        if outer.done():
+            if not fut.cancelled():
                 # Mark exception retrieved.
                 fut.exception()
             return
-        if fut._state == futures._CANCELLED:
+
+        if fut.cancelled():
             res = futures.CancelledError()
             if not return_exceptions:
                 outer.set_exception(res)
@@ -693,7 +710,7 @@ def shield(arg, loop=None):
         except CancelledError:
             res = None
     """
-    inner = async(arg, loop=loop)
+    inner = ensure_future(arg, loop=loop)
     if inner.done():
         # Shortcut.
         return inner
@@ -702,9 +719,11 @@ def shield(arg, loop=None):
 
     def _done_callback(inner):
         if outer.cancelled():
-            # Mark inner's result as retrieved.
-            inner.cancelled() or inner.exception()
+            if not inner.cancelled():
+                # Mark inner's result as retrieved.
+                inner.exception()
             return
+
         if inner.cancelled():
             outer.cancel()
         else:
